/*
 Copyright (c) 2013 yvt
 Modified by Paratrooper
 
 This file is part of OpenSpades.

 OpenSpades is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenSpades is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenSpades.  If not, see <http://www.gnu.org/licenses/>.

 */

 namespace spades {
	class ThirdPersonRifleSkin:
	IToolSkin, IThirdPersonToolSkin, IWeaponSkin, IWeaponSkin2 {
		private float sprintState;
		private float raiseState;
		private Vector3 teamColor;
		private bool muted = true;
		private Matrix4 originMatrix;
		private float aimDownSightState;
		private float readyState;
		private bool reloading;
		private float reloadProgress;
		private int ammo, clipSize;

		private float environmentRoom;
		private float environmentSize;
		private float environmentDistance;
		private Vector3 soundOrigin;

		float SprintState {
			set { sprintState = value; }
		}

		float RaiseState {
			set { raiseState = value; }
		}

		bool IsMuted {
			set { muted = value; }
		}

		Vector3 TeamColor {
			set { teamColor = value; }
		}

		Matrix4 OriginMatrix {
			set { originMatrix = value; }
		}

		float PitchBias {
			get { return 0.f; }
		}

		float AimDownSightState {
			set { aimDownSightState = value; }
		}

		bool IsReloading {
			set { reloading = value; }
		}
		float ReloadProgress {
			set { reloadProgress = value; }
		}
		int Ammo {
			set { ammo = value; }
		}
		int ClipSize {
			set { clipSize = value; }
		}

		float ReadyState {
			set { readyState = value; }
		}

		// IWeaponSkin2
		void SetSoundEnvironment(float room, float size, float distance) {
			environmentRoom = room;
			environmentSize = size;
			environmentDistance = distance;
		}
		Vector3 SoundOrigin {
			set { soundOrigin = value; }
		}

		private Renderer@ renderer;
		private AudioDevice@ audioDevice;
		private Model@ Body;
		private Model@ Barrel;
		private Model@ Bolt;
		private Model@ Handguard;
		private Model@ Stock;
		private Model@ Mag1;
		private Model@ Mag2;
		private ConfigItem ss("ss", "0.15");
		private ConfigItem xx("xx", "0");
		private ConfigItem yy("yy", "0");
		private ConfigItem zz("zz", "0");
		private AudioChunk@[] fireSounds(3);
		private AudioChunk@ fireFarSound;
		private AudioChunk@ fireStereoSound;
		private AudioChunk@ reloadSound;

		

		ThirdPersonRifleSkin(Renderer@ r, AudioDevice@ dev) {
			@renderer = r;
			@audioDevice = dev;
			@Body = renderer.RegisterModel("Models/Weapons/Rifle/Body.kv6");
			@Barrel = renderer.RegisterModel("Models/Weapons/Rifle/Barrel.kv6");
			@Bolt = renderer.RegisterModel("Models/Weapons/Rifle/Bolt.kv6");
			@Handguard = renderer.RegisterModel("Models/Weapons/Rifle/Handguard.kv6");
			@Stock = renderer.RegisterModel("Models/Weapons/Rifle/Stock.kv6");
			@Mag1 = renderer.RegisterModel("Models/Weapons/Rifle/Mag1.kv6");
			@Mag2 = renderer.RegisterModel("Models/Weapons/Rifle/Mag2.kv6");	

			@fireFarSound = dev.RegisterSound
				("Sounds/Weapons/Rifle/FireFar.wav");
			@fireStereoSound = dev.RegisterSound
				("Sounds/Weapons/Rifle/FireStereo.wav");
			@reloadSound = dev.RegisterSound
				("Sounds/Weapons/Rifle/Reload.wav");
		}

		void Update(float dt) {
		}

		void WeaponFired(){
			if(!muted){
				Vector3 origin = soundOrigin;
				AudioParam param;

				param.volume = 1.f;
				param.referenceDistance = 26.f;
				audioDevice.Play(fireFarSound, origin, param);
				param.referenceDistance = 4.f;
				param.volume = 1.f;
				audioDevice.Play(fireStereoSound, origin, param);
			}
		}

		void ReloadingWeapon() {
			if(!muted){
				Vector3 origin = soundOrigin;
				AudioParam param;
				param.volume = 0.2f;
				audioDevice.Play(reloadSound, origin, param);
			}
		}

		void ReloadedWeapon() {
		}

		void AddToScene() {
			Matrix4 mat = CreateScaleMatrix(0.05f/3.f);
			mat = mat * CreateScaleMatrix(-1.f, -1.f, 1.f);
			mat = CreateTranslateMatrix(0.4f, -0.6f, -0.1f) * mat;

			
			
			ModelRenderParam param;
			param.matrix = originMatrix * mat
			* CreateScaleMatrix(0.2F)
			* CreateTranslateMatrix(0.0F, 100.0F, -25.0F)
			* CreateRotateMatrix( Vector3(1.0, 0.0, 0.0), 0.0F )
      * CreateRotateMatrix( Vector3(0.0, 1.0, 0.0), 0.0F )
      * CreateRotateMatrix( Vector3(0.0, 0.0, 1.0), -3.1F );
			renderer.AddModel(Body, param);
			renderer.AddModel(Barrel, param);
			renderer.AddModel(Handguard, param);
			renderer.AddModel(Stock, param);
			renderer.AddModel(Bolt, param);
			renderer.AddModel(Mag1, param);
			renderer.AddModel(Mag2, param);
		}
	}

	IWeaponSkin@ CreateThirdPersonRifleSkin(Renderer@ r, AudioDevice@ dev) {
		return ThirdPersonRifleSkin(r, dev);
	}
}
